package securityagent;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.security.AllPermission;
import java.util.ArrayList;
import java.util.List;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.condpermadmin.BundleLocationCondition;
import org.osgi.service.condpermadmin.ConditionalPermissionAdmin;
import org.osgi.service.condpermadmin.ConditionalPermissionInfo;
import org.osgi.service.condpermadmin.ConditionalPermissionUpdate;

public class SecurityAgentPOC implements BundleActivator {
	/*
	 * (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bundleContext) throws Exception {
		ConditionalPermissionAdmin admin = null;
		ServiceReference<ConditionalPermissionAdmin> ref = bundleContext.getServiceReference(ConditionalPermissionAdmin.class);
		if (ref != null) {
		    admin = bundleContext.getService(ref);
		}
		if (admin != null) {
		    //read system wide, user configured permission properties from the file specified in this system property
		    String policyFileLocation = System.getProperty("osgi.policyFile");
		    String star = " \"*\"";
		    //give ALL permission to this agent
		    ConditionalPermissionInfo info_all_permission_self = admin.newConditionalPermissionInfo(ConditionalPermissionInfo.ALLOW + "{[" + BundleLocationCondition.class.getName() + " \"" + bundleContext.getBundle().getLocation() + "\"]("+AllPermission.class.getName() + star + star + ")}");
		    //update the conditional permission table in a single transaction
		    ConditionalPermissionUpdate update = admin.newConditionalPermissionUpdate();
		    List<ConditionalPermissionInfo> infos = update.getConditionalPermissionInfos();
		    infos.clear();
		    infos.add(info_all_permission_self);
		    for (ConditionalPermissionInfo info : getPermissionInfosFrom(new File(policyFileLocation), admin)) {
		        infos.add(info);
		    }
		    update.commit();		    
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bundleContext) throws Exception {
	}

	private List<ConditionalPermissionInfo> getPermissionInfosFrom(File policyFile, ConditionalPermissionAdmin admin) throws IOException {
	    List<String> unparsedPermissionInfos = readPermissionInfos(policyFile);
	    return parsePermissionInfos(unparsedPermissionInfos, admin);
	}

    private List<ConditionalPermissionInfo> parsePermissionInfos(List<String> unparsedPermissionInfos, ConditionalPermissionAdmin admin) {
        List<ConditionalPermissionInfo> parsed = new ArrayList<ConditionalPermissionInfo>();
        for (String info : unparsedPermissionInfos) {
            parsed.add(admin.newConditionalPermissionInfo(info));
        }
        return parsed;
    }

    private List<String> readPermissionInfos(File file) throws IOException {
        List<String> read = new ArrayList<String>();
        BufferedReader reader = new BufferedReader(new FileReader(file));
        String line = null;
        StringBuilder accumulator = new StringBuilder();
        while ((line = reader.readLine()) != null) {
            if (line.contains("};")) {
                accumulator.append(line);
                read.add(accumulator.toString());
                accumulator = new StringBuilder();
            } else {
                accumulator.append(line);
            }
        }
        return read;
    }
	
}
